/* ------------------------------------------------------------ */
/* -- Authors:						     -- */
/* --	Prasad Deshpande				     -- */
/* --	Avinash Sodani					     -- */
/* --							     -- */
/* --	Copyright (c) 1995  Computer Sciences Department     -- */
/* --	University of Wisconsin, Madison		     -- */
/* --	All rights reserved				     --	*/
/* ------------------------------------------------------------ */

#include <rpc/rpc.h>
#define _BOOL_H_

#include <StubPreamble.h>
#include "rpcinterface.h"
#include "timeval.h"
#include <stdio.h>
#include <assert.h>

#include <javaString.h>
#include "sockaddr_in.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include "proctab.h"

bool_t xdr_in(XDR *, char *);
bool_t xdr_out(XDR *, char *);

struct Hfunc *Inproc, *Outproc;
XDR*  Xdrp;

long rpcinterface_callrpc(struct Hrpcinterface * this,  Hjava_lang_String *host, long prognum,
			  long versnum, long procnum, struct Hfunc *inproc,
			  struct Hjava_lang_Object * in,	 struct Hfunc *outproc, 
			  struct Hjava_lang_Object *out)
{
    
    char *hostname;
    
    Inproc = inproc;
    Outproc = outproc;
    
    hostname  = makeCString(host);
    assert(hostname);
    
    return callrpc(hostname, prognum, versnum, procnum, xdr_in, (char *)in, xdr_out, (char *)out); 
}


bool_t xdr_in(XDR *xdrs, char *in) {

    Xdrp = xdrs;
    return execute_java_dynamic_method(0, (HObject *)Inproc, "xdr_func", "(Ljava/lang/Object;)Z", 
				       (Hjava_lang_Object *)in);
}

bool_t xdr_out(XDR *xdrs, char *out) {
    
    Xdrp = xdrs;
    return execute_java_dynamic_method(0, (HObject *)Outproc, "xdr_func", "(Ljava/lang/Object;)Z", 
				       (Hjava_lang_Object *)out);
}

long rpcinterface_clnt_call(struct Hrpcinterface *this, long clnt, long procnum, struct Hfunc *inproc, 
			    struct Hjava_lang_Object *in, struct Hfunc *outproc, 
			    struct Hjava_lang_Object *out, struct Htimeval *timeout)
{
    struct timeval tout;
    Classtimeval *ctval = unhand(timeout);
    assert(ctval);
    
    Inproc = inproc;
    Outproc = outproc;
    
    tout.tv_sec = ctval->tv_sec;
    tout.tv_usec = ctval->tv_usec;
    
    return clnt_call((CLIENT *)clnt, procnum, xdr_in, (caddr_t) in, xdr_out, (caddr_t) out,
		     tout);
}

long rpcinterface_clnt_create(struct Hrpcinterface *this, struct Hjava_lang_String *host, long prognum, 
			      long versnum, struct Hjava_lang_String *nettype)
{
    char *hostname, *nettypename;
    
    hostname = makeCString(host);
    assert(hostname);
    nettypename = makeCString(nettype);
    assert(nettypename);
    
    return (long) clnt_create(hostname, prognum, versnum, nettypename);
}

void rpcinterface_clnt_destroy(struct Hrpcinterface *this,long clnt)
{
    clnt_destroy((CLIENT *)clnt);
}

void rpcinterface_clnt_pcreateerror(struct Hrpcinterface *this, struct Hjava_lang_String *s)
{
    char *errs = makeCString(s);
    
    clnt_pcreateerror(errs);
}

void rpcinterface_clnt_perrno(struct Hrpcinterface *this, long stat)
{
    clnt_perrno(stat);
}

void rpcinterface_clnt_perror(struct Hrpcinterface *this, long clnt,struct Hjava_lang_String *s)
{
    char *errs = makeCString(s);
    
    clnt_perror((CLIENT *)clnt, errs);
}



ProcTab_el ProcTab[MAXNUMPROCS];

int	   proctab_index=0;   /* present index into the proc table */





      /***     get_myaddress ***/

void rpcinterface_get_myaddress(struct Hrpcinterface *this,
			        struct Hsockaddr_in *addr_ptr)
{
 Classsockaddr_in *sockptr = unhand(addr_ptr);
 struct sockaddr_in  addr;

 get_myaddress(&addr);

 sockptr->sin_family =  addr.sin_family;
 sockptr->sin_port = addr.sin_port; 
 sockptr->in_addr = addr.sin_addr.S_un.S_addr;

}

	/***   getnetname  ***/

long rpcinterface_getnetname(struct Hrpcinterface *this,
			     HArrayOfChar *jnetname,long length)
{
 char *netname = (char *) unhand(jnetname)->body;

 if(!getnetname(netname))
 {
    printf("Some error occured %d\n", errno);
    return(FALSE);
 }
 
 return(TRUE);
}

      /**** getrpcport ****/

long rpcinterface_getrpcport(struct Hrpcinterface * this,
			 Hjava_lang_String *jhostname,long prognum ,
			 long versnum,long proto)
{
 
  char *hostname; 
  hostname = makeCString(jhostname);
  assert(hostname);

/*  return((long)getrpcport(hostname,(int)prognum, (int)versnum, (int)proto)); */
/* check-- getrpcport is not defined on solaris */
  
  return 0;
}


     /***** registerrpc *****/

long rpcinterface_registerrpc(struct Hrpcinterface * this, long prognum,
		long versnum, long procnum, struct Hservfunc *procname,
		struct Hfunc *inproc, struct Hfunc *outproc, struct Hjava_lang_Object *Inobj,
		struct Hjava_lang_Object *Outobj)
{
 int retval;

 if(proctab_index >= MAXNUMPROCS)
     return( -1);
 
 printf("called C registerrpc \n");
 retval = registerrpc(prognum, versnum, procnum,
		      cfunc0, xdr_in0,
		      xdr_out0);
 if(retval == -1)
   return(-1);
 
 ProcTab[proctab_index].prognum = prognum;
 ProcTab[proctab_index].versnum = versnum;
 ProcTab[proctab_index].procnum = procnum;
 ProcTab[proctab_index].procname = procname;
 ProcTab[proctab_index].inproc = inproc;
 ProcTab[proctab_index].outproc = outproc;
 ProcTab[proctab_index].Inobj = Inobj;
 ProcTab[proctab_index].Outobj = Outobj;
 proctab_index++;

 return(0);
}

     /*** svc_run ***/

void rpcinterface_svc_run(struct Hrpcinterface *this)
{
 printf("called C svc_run \n");
  svc_run();
}

